﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Diagnostics;
using System.Linq;
using System.Threading;
using System.Windows;
using Windows.Devices.Bluetooth.Advertisement;
using Windows.Storage.Streams;

namespace iBeaconScanner
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        private BluetoothLEAdvertisementWatcher advWatcher;

        /*　定数　*/
        public static int CompanyID_Apple = 76;
        public static int Length_of_iBeacon_Advertise_Packet = 23;
        public static int OKnum = 1;
        public static TimeSpan TIMEOUT = new TimeSpan(0,0,3);//3秒離れてたらタイムアウト

        static int RUcnt, NUcnt;

        static HashSet<string> RU_uuid = new HashSet<string>(); //RUのハッシュセット
        //static HashSet<string> Around_uuid = new HashSet<string>(); //周囲のuuidのハッシュセット
        Dictionary<string,DateTimeOffset> Around_uuid = new Dictionary<string, DateTimeOffset>(); //周囲のuuidのハッシュセット


        public MainWindow()
        {
            InitializeComponent();

            /* 事前登録 */
            RU_uuid.Add("48534442-4c45-4144-1234-dcba12345678");
            RU_uuid.Add("48534442-4c45-4144-1234-abcd12345678");

            this.advWatcher = new BluetoothLEAdvertisementWatcher();

            // https://blogs.msdn.microsoft.com/shozoa/2016/02/28/windows-10-bluetooth/
            // インターバルがゼロのままだと、CPU負荷が高くなりますので、適切な間隔(SDK サンプルでは 1秒)に指定しないと、アプリの動作に支障をきたすことになります。
            this.advWatcher.SignalStrengthFilter.SamplingInterval = TimeSpan.FromMilliseconds(1000);

            // rssi >= -60のときスキャンする
            //this.advWatcher.SignalStrengthFilter.InRangeThresholdInDBm = -60;

            this.advWatcher.Received += this.Watcher_Received;

            // スキャン開始
            this.advWatcher.Start();
        }

        private async void Watcher_Received(BluetoothLEAdvertisementWatcher sender, BluetoothLEAdvertisementReceivedEventArgs args)
        {
            await this.Dispatcher.InvokeAsync(() =>
            {

                foreach (var manufacturerData in args.Advertisement.ManufacturerData)
                {
                    /*
                    Debug.WriteLine($"{args.Timestamp} {args.Advertisement.LocalName}:{args.BluetoothAddress}");
                    Debug.WriteLine($"RSSI={args.RawSignalStrengthInDBm}");
                    Debug.WriteLine($"length:{manufacturerData.Data.Length}");
                    Debug.WriteLine($"companyId :{manufacturerData.CompanyId}");
                    Debug.WriteLine($"Ad type :{args.AdvertisementType}");
                    Debug.WriteLine($"--------------");
                    */

                    // mutex.WaitOne();

                    if (manufacturerData.CompanyId != CompanyID_Apple)
                    {
                        continue;
                    }
                    if (manufacturerData.Data.Length != Length_of_iBeacon_Advertise_Packet)
                    {
                        continue;
                    }

                    Debug.WriteLine($"-------------");
                    Debug.WriteLine($"{args.Timestamp} RSSI={args.RawSignalStrengthInDBm} BD:{args.BluetoothAddress}");
                    // Debug.WriteLine($"{args.AdvertisementType}, {manufacturerData.CompanyId}");

                    using (var reader = DataReader.FromBuffer(manufacturerData.Data))
                    {
                        reader.ByteOrder = ByteOrder.BigEndian;// BigEndian:順方向にパケットを保存

                        /* 0215 で始まればiBeacon */  //http://d.hatena.ne.jp/shu223/20141208/1418051009
                        var indicator = reader.ReadByte();
                        var length = reader.ReadByte();
                        if (indicator != 0x02 && length != 0x15)
                        {
                            continue; //iBeacon でないAdvertise Packet
                        }

                        Debug.WriteLine("#####This is iBeacon Packet!!!");

                        // Parse advertise packet.
                        var uuidA = reader.ReadInt32();
                        var uuidB = reader.ReadInt16();
                        var uuidC = reader.ReadInt16();
                        var uuidD = new byte[8];
                        reader.ReadBytes(uuidD);
                        var major = reader.ReadUInt16();
                        var minor = reader.ReadUInt16();
                        var power = reader.ReadByte();

                        var proximity_uuid = new Guid(uuidA, uuidB, uuidC, uuidD);
                       
                        /* RUのハッシュセットを作成 *//*
                        if( !RU_uuid.Contains(proximity_uuid.ToString() ) )
                        {
                            RU_uuid.Add(proximity_uuid.ToString() );
                        }
                        foreach (var RU in RU_uuid)
                        {
                            Debug.WriteLine($"  RU is {RU}");
                            Console.WriteLine($"  RU is {RU}");
                        }
                        Console.WriteLine($"------------");
                        */

                        /* 周囲のuuidのDictionaryを作成 */
                        if (!Around_uuid.Keys.Contains(proximity_uuid.ToString())) //新しいUUIDなら追加 ????????
                        {
                            Around_uuid.Add(proximity_uuid.ToString(),args.Timestamp);
                            Debug.WriteLine($"New!");
                            Debug.WriteLine($"UUID: {proximity_uuid}, major: {major}, minor: {minor}, power: {power}");
                        }
                        else //既にあるUUIDなら時間更新
                        {
                            Around_uuid[proximity_uuid.ToString()] = args.Timestamp;
                            Debug.WriteLine($"Time Refreshed...");
                        }

                        /* Timestamp が古いものを削除 */
                        DateTime dt = DateTime.Now;
                        var removeTargetLst = Around_uuid.Where(kv => (dt - kv.Value) < TIMEOUT).ToList();
                        foreach (var item in removeTargetLst)
                        {
                            Around_uuid.Remove(item.Key);  // ここで削除する。Keyを指定して削除出来る。
                        }

                        /* 周囲にあるUUID */
                        foreach (var some in Around_uuid)
                        {
                            Debug.WriteLine($"UUID:{some.Key} ,time:{some.Value}");
                            this.textBox.Text = this.textBox.Text + some.Value + ":" + some.Key +"\r\n";
                        }

                        /* 判定フェーズ↓ */
                        RUcnt = 0;
                        NUcnt = 0;
                        foreach (var N in RU_uuid)
                        {
                            if (Around_uuid.Keys.Contains(N))
                            {
                                RUcnt++;
                            }
                            else { NUcnt++; }
                        }
                        this.textBox.Text = this.textBox.Text + "RU:" + RUcnt + "人,　NU" + NUcnt + "人" + "\r\n";

                        if (RUcnt >= OKnum && RUcnt * 3 > NUcnt)//安全かどうかの判定
                        {
                            this.textBox.Text = this.textBox.Text + "You are YOU!!" + "\r\n";
                            // watcher.Stop(); 
                        }
                        /* 判定フェーズ↑ */

                    }
                }

            });   
        }
    }
}
